<?php

namespace Research\Chapter\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Research\Chapter\Http\Requests\ChapterRequest;
use Research\Chapter\Interfaces\ChapterRepositoryInterface;
use Research\Chapter\Models\Chapter;
use Research\Chapter\Forms\Chapter as Form;

/**
 * APIController  class for chapter.
 */
class ChapterAPIController extends BaseController
{

    /**
     * Initialize chapter resource controller.
     *
     * @param type ChapterRepositoryInterface $chapter
     *
     * @return null
     */
    public function __construct(ChapterRepositoryInterface $chapter)
    {
        parent::__construct();
        $this->repository = $chapter;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Research\Chapter\Repositories\Criteria\ChapterResourceCriteria::class);
    }

    /**
     * Display a list of chapter.
     *
     * @return Response
     */
    public function index(ChapterRequest $request)
    {
        return $this->repository
            ->setPresenter(\Research\Chapter\Repositories\Presenter\ChapterPresenter::class)
            ->paginate();
    }

    /**
     * Display chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return Response
     */
    public function show(ChapterRequest $request, Chapter $chapter)
    {
        return $chapter->setPresenter(\Research\Chapter\Repositories\Presenter\ChapterListPresenter::class);
        ;
    }

    /**
     * Create new chapter.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ChapterRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('chapter::chapter.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('chapter/chapter/' . $chapter->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('chapter/chapter');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the chapter.
     *
     * @param Request $request
     * @param Model   $chapter
     *
     * @return Response
     */
    public function update(ChapterRequest $request, Chapter $chapter)
    {
        try {
            $data = $request->all();

            $chapter->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('chapter::chapter.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('chapter/chapter/' . $chapter->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('chapter/chapter/' . $chapter->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the chapter.
     *
     * @param Model   $chapter
     *
     * @return Response
     */
    public function destroy(ChapterRequest $request, Chapter $chapter)
    {
        try {
            $chapter->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('chapter::chapter.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('chapter/chapter/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('chapter/chapter/' . $chapter->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
